@@ -1,21 +1,14 @@
-# PYTHON_ARGCOMPLETE_OK
-# SPDX-FileCopyrightText: 2023 geisserml <geisserml@gmail.com>
+# SPDX-FileCopyrightText: 2024 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
-import os
 import sys
-import logging
 import argparse
 import importlib
-import pypdfium2 as pdfium  # yes this is legal in __main__
-import pypdfium2.internal as pdfium_i
-from pypdfium2.version import *
-
-try:
-    import argcomplete
-except ImportError:
-    argcomplete = None
+from pypdfium2.version import PYPDFIUM_INFO, PDFIUM_INFO
+from pypdfium2._cli._parsers import setup_logging
 
+from pypdfium2_raw.bindings import _libs_info
+pdfium_path = _libs_info["pdfium"]["path"]
 
 SubCommands = {
     "arrange":        "rearrange/merge documents",
@@ -37,12 +30,13 @@ def get_parser():
     
     main_parser = argparse.ArgumentParser(
         prog = "pypdfium2",
+        formatter_class = argparse.RawTextHelpFormatter,
         description = "Command line interface to the pypdfium2 library (Python binding to PDFium)",
     )
     main_parser.add_argument(
         "--version", "-v",
         action = "version",
-        version = f"pypdfium2 {V_PYPDFIUM2} (libpdfium {V_LIBPDFIUM}, origin: {V_BUILDNAME}, flags: {['V8', 'XFA'] if V_PDFIUM_IS_V8 else []})",
+        version = f"pypdfium2 {PYPDFIUM_INFO}\n" f"pdfium {PDFIUM_INFO} at {pdfium_path}"
     )
     subparsers = main_parser.add_subparsers(dest="subcommand")
     
@@ -50,23 +44,9 @@ def get_parser():
         subparser = subparsers.add_parser(name, description=help, help=help)
         CmdToModule[name].attach(subparser)
     
-    if argcomplete:
-        argcomplete.autocomplete(main_parser)
-    
     return main_parser
 
 
-def setup_logging():
-    
-    pdfium_i.DEBUG_AUTOCLOSE.value = bool(int( os.environ.get("DEBUG_AUTOCLOSE", 0) ))
-    
-    lib_logger = logging.getLogger("pypdfium2")
-    lib_logger.addHandler(logging.StreamHandler())
-    lib_logger.setLevel(logging.DEBUG)
-    
-    pdfium.PdfUnspHandler().setup()
-
-
 def api_main(raw_args=sys.argv[1:]):
     
     parser = get_parser()
