@@ -1,4 +1,4 @@
-# SPDX-FileCopyrightText: 2023 geisserml <geisserml@gmail.com>
+# SPDX-FileCopyrightText: 2024 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
 import atexit
@@ -8,11 +8,22 @@ import pypdfium2.internal as pdfium_i
 
 
 def init_lib():
-    # NOTE PDFium developers plan changes to the initialisation API (see https://crbug.com/pdfium/1446)
     assert not pdfium_i.LIBRARY_AVAILABLE
     if pdfium_i.DEBUG_AUTOCLOSE:
         print("Initialize PDFium (auto)", file=sys.stderr)
-    pdfium_c.FPDF_InitLibrary()
+    
+    # PDFium init API may change in the future: https://crbug.com/pdfium/1446
+    # NOTE Technically, FPDF_InitLibrary() would be sufficient for our purposes, but since it's formally marked for deprecation, don't use it to be on the safe side. Also, avoid experimental config versions that might not be promoted to stable.
+    config = pdfium_c.FPDF_LIBRARY_CONFIG(
+        version = 2,
+        m_pUserFontPaths = None,
+        m_pIsolate = None,
+        m_v8EmbedderSlot = 0,
+        # m_pPlatform = None,  # v3
+        # m_RendererType = pdfium_c.FPDF_RENDERERTYPE_AGG,  # v4
+    )
+    pdfium_c.FPDF_InitLibraryWithConfig(config)
+    
     pdfium_i.LIBRARY_AVAILABLE.value = True
 
 
